/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.registry.overlay;

import java.util.HashSet;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration.ContainerState;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.configuration.ContainerConfigurationListener;
import org.ow2.petals.microkernel.api.configuration.DomainConfiguration;
import org.ow2.petals.microkernel.api.configuration.SubDomainConfiguration;
import org.ow2.petals.microkernel.api.configuration.exception.ConfigurationException;
import org.ow2.petals.microkernel.api.configuration.exception.ContainerAlreadyExistException;
import org.ow2.petals.microkernel.api.configuration.exception.ContainerDoesNotExistException;
import org.ow2.petals.microkernel.api.configuration.exception.InvalidContainerConfigurationProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.InvalidSubdomainConfigurationProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.MissingServerPropertyException;
import org.ow2.petals.microkernel.api.configuration.exception.NoContainerConfigurationProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.NoDomainConfigurationProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.NoDomainNameProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.NoServerPropertiesProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.NoSubdomainConfigurationProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.NoSubdomainNameProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.NoTopologyProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.SubdomainAlreadyExistException;
import org.ow2.petals.microkernel.api.configuration.exception.SubdomainDoesNotExistException;
import org.ow2.petals.microkernel.api.configuration.exception.SubdomainUnknownException;
import org.ow2.petals.topology.generated.Topology;

/**
 * An implementation of {@link ConfigurationService} used to unit test {@link DynamicTopologyListener}.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class DynamicTopologyListenerTestCfgSvc implements ConfigurationService {

    private final Set<ContainerConfiguration> containerConfig = new HashSet<ContainerConfiguration>();

    private final boolean waitOnAdd;

    private final ContainerConfiguration currentContainerConfig;

    public DynamicTopologyListenerTestCfgSvc(final boolean waitOnAdd,
            final ContainerConfiguration currentContainerConfig, final ContainerConfiguration remoteContainerConfig) {
        this.waitOnAdd = waitOnAdd;
        this.currentContainerConfig = currentContainerConfig;
        if (remoteContainerConfig != null) {
            this.containerConfig.add(remoteContainerConfig);
        }
    }

    @Override
    public ContainerConfiguration getContainerConfiguration() {
        return this.currentContainerConfig;
    }

    @Override
    public Map<String, String> getAllServerProperties() {
        return null;
    }

    @Override
    public Properties getServerProperties() {
        return null;
    }

    @Override
    public Set<ContainerConfiguration> getContainersConfiguration() {
        return this.containerConfig;
    }

    @Override
    public Set<SubDomainConfiguration> getSubDomainsConfiguration() {
        return null;
    }

    @Override
    public SubDomainConfiguration getSubDomainConfiguration() {
        return null;
    }

    @Override
    public SubDomainConfiguration getSubDomainConfiguration(final String subdomainName) {
        return null;
    }

    @Override
    public DomainConfiguration getDomainConfiguration() {
        return null;
    }

    @Override
    public Topology getInitialLocalTopology() throws ConfigurationException {
        return null;
    }

    @Override
    public void setContainerState(final ContainerState state) {
        // NOP
    }

    @Override
    public ContainerConfiguration getContainerConfiguration(final String containerName) {
        return null;
    }

    @Override
    public void addContainerConfiguration(final ContainerConfiguration addedContainer)
            throws NoContainerConfigurationProvidedException, InvalidContainerConfigurationProvidedException,
            ContainerAlreadyExistException, SubdomainUnknownException {
        this.containerConfig.add(addedContainer);
        if (this.waitOnAdd) {
            synchronized (this) {
                this.notifyAll();
            }
        }
    }

    @Override
    public void removeContainerConfiguration(final ContainerConfiguration removedContainer)
            throws NoContainerConfigurationProvidedException, ContainerDoesNotExistException,
            InvalidContainerConfigurationProvidedException {
        this.containerConfig.remove(removedContainer);
        if (!this.waitOnAdd) {
            synchronized (this) {
                this.notifyAll();
            }
        }
    }

    @Override
    public void addSubdomainConfiguration(final SubDomainConfiguration addedSubdomain)
            throws SubdomainAlreadyExistException, NoSubdomainConfigurationProvidedException,
            InvalidSubdomainConfigurationProvidedException {
        // NOP
    }

    @Override
    public void removeSubdomainConfiguration(final SubDomainConfiguration removedSubdomain)
            throws NoSubdomainConfigurationProvidedException, SubdomainDoesNotExistException,
            InvalidSubdomainConfigurationProvidedException {
        // NOP
    }

    @Override
    public void loadConfiguration(final Properties serverLocalProperties, final Topology topology)
            throws NoTopologyProvidedException, NoServerPropertiesProvidedException, MissingServerPropertyException,
            ConfigurationException {
        // NOP
    }

    @Override
    public void updateTopology(final Topology topology) throws NoTopologyProvidedException,
            InvalidSubdomainConfigurationProvidedException, ConfigurationException {
        // NOP
    }

    @Override
    public void reconfigureDomain(final DomainConfiguration newDomainConfiguration)
            throws NoDomainConfigurationProvidedException, NoDomainNameProvidedException {
        // NOP
    }

    @Override
    public void reconfigureSubdomain(final SubDomainConfiguration newSubdomainConfiguration)
            throws NoSubdomainConfigurationProvidedException, NoSubdomainNameProvidedException,
            SubdomainAlreadyExistException {
        // NOP
    }

    @Override
    public void reconfigureContainer(final SubDomainConfiguration newSubdomainConfiguration)
            throws NoSubdomainConfigurationProvidedException, NoSubdomainNameProvidedException {
        // NOP
    }

    @Override
    public boolean isSecurityTopologyPassphraseOk(final String securityPassPhrase) {
        return false;
    }

    @Override
    public void registerContainerConfigurationListener(ContainerConfigurationListener listener) {
        // NOP
    }

    @Override
    public void deregisterContainerConfigurationListener(ContainerConfigurationListener listener) {
        // NOP
    }

    @Override
    public void persistTopology(Topology topology) throws ConfigurationException {
        // NOP
    }

}
